from typing import Any, Dict, List, cast

from typing_extensions import Literal, NotRequired, TypedDict

from resend import request
from resend.domains._domain import Domain

TlsOptions = Literal["enforced", "opportunistic"]


class _ListResponse(TypedDict):
    data: List[Domain]
    """
    A list of domain objects
    """


class Domains:

    class ListResponse(_ListResponse):
        """
        ListResponse type that wraps a list of domain objects

        Attributes:
            data (List[Domain]): A list of domain objects
        """

    class UpdateParams(TypedDict):
        id: str
        """
        The domain ID.
        """
        click_tracking: NotRequired[bool]
        """
        Track clicks within the body of each HTML email.
        """
        open_tracking: NotRequired[bool]
        """
        Track the open rate of each email.
        """
        tls: NotRequired[TlsOptions]
        """
        default: "opportunistic"
        opportunistic: Opportunistic TLS means that it always attempts to make a
        secure connection to the receiving mail server.
        If it can't establish a secure connection, it sends the message unencrypted.

        enforced: Enforced TLS on the other hand, requires that the email
        communication must use TLS no matter what.
        If the receiving server does not support TLS, the email will not be sent.
        """

    class CreateParams(TypedDict):
        name: str
        """
        The domain name.
        """
        region: NotRequired[str]
        """
        The region where emails will be sent from.
        Possible values: us-east-1' | 'eu-west-1' | 'sa-east-1' | 'ap-northeast-1'
        """

    @classmethod
    def create(cls, params: CreateParams) -> Domain:
        """
        Create a domain through the Resend Email API.
        see more: https://resend.com/docs/api-reference/domains/create-domain

        Args:
            params (CreateParams): The domain creation parameters

        Returns:
            Domain: The new domain object
        """
        path = "/domains"
        resp = request.Request[Domain](
            path=path, params=cast(Dict[Any, Any], params), verb="post"
        ).perform_with_content()
        return resp

    @classmethod
    def update(cls, params: UpdateParams) -> Domain:
        """
        Update an existing domain.
        see more: https://resend.com/docs/api-reference/domains/update-domain

        Args:
            params (UpdateParams): The domain update parameters

        Returns:
            Domain: The updated domain object
        """
        path = f"/domains/{params['id']}"
        resp = request.Request[Domain](
            path=path, params=cast(Dict[Any, Any], params), verb="patch"
        ).perform_with_content()
        return resp

    @classmethod
    def get(cls, domain_id: str) -> Domain:
        """
        Retrieve a single domain for the authenticated user.
        see more: https://resend.com/docs/api-reference/domains/get-domain

        Args:
            domain_id (str): The domain ID

        Returns:
            Domain: The domain object
        """
        path = f"/domains/{domain_id}"
        resp = request.Request[Domain](
            path=path, params={}, verb="get"
        ).perform_with_content()
        return resp

    @classmethod
    def list(cls) -> ListResponse:
        """
        Retrieve a list of domains for the authenticated user.
        see more: https://resend.com/docs/api-reference/domains/list-domains

        Returns:
            ListResponse: A list of domain objects
        """
        path = "/domains"
        resp = request.Request[_ListResponse](
            path=path, params={}, verb="get"
        ).perform_with_content()
        return resp

    @classmethod
    def remove(cls, domain_id: str) -> Domain:
        """
        Remove an existing domain.
        see more: https://resend.com/docs/api-reference/domains/delete-domain

        Args:
            domain_id (str): The domain ID

        Returns:
            Domain: The removed domain object
        """
        path = f"/domains/{domain_id}"
        resp = request.Request[Domain](
            path=path, params={}, verb="delete"
        ).perform_with_content()
        return resp

    @classmethod
    def verify(cls, domain_id: str) -> Domain:
        """
        Verify an existing domain.
        see more: https://resend.com/docs/api-reference/domains/verify-domain

        Args:
            domain_id (str): The domain ID

        Returns:
            Domain: The verified domain object
        """
        path = f"/domains/{domain_id}/verify"
        resp = request.Request[Domain](
            path=path, params={}, verb="post"
        ).perform_with_content()
        return resp



from typing_extensions import TypedDict


class Contact(TypedDict):
    id: str
    """
    The contact id.
    """
    email: str
    """
    The email of the contact.
    """
    first_name: str
    """
    The first name of the contact.
    """
    last_name: str
    """
    The last name of the contact.
    """
    created_at: str
    """
    The timestamp of the contact.
    """
    unsubscribed: bool
    """
    The unsubscribed status of the contact.
    """
    deleted: bool
    """
    Wether the contact is deleted or not.
    """



from typing import List, Union

from typing_extensions import TypedDict

# Uses functional typed dict syntax here in order to support "from" reserved keyword
_FromParam = TypedDict(
    "_FromParam",
    {
        "from": str,
    },
)


class Broadcast(_FromParam):
    object: str
    """
    The object type, which is always "broadcast".
    """
    id: str
    """
    The unique identifier of the broadcast.
    """
    audience_id: str
    """
    The unique identifier of the audience.
    """
    name: str
    """
    The name of the broadcast.
    """
    subject: str
    """
    The subject of the broadcast.
    """
    reply_to: Union[List[str], str]
    """
    The reply-to email address.
    """
    preview_text: str
    """
    The preview text of the broadcast.
    """
    status: str
    """
    The status of the broadcast.
    """
    created_at: str
    """
    The date and time the broadcast was created.
    """
    scheduled_at: str
    """
    The date and time the broadcast is scheduled to be sent.
    """
    sent_at: str
    """
    The date and time the broadcast was sent.
    """



from typing import List, Union

from typing_extensions import TypedDict

# Uses functional typed dict syntax here in order to support "from" reserved keyword
_FromParam = TypedDict(
    "_FromParam",
    {
        "from": str,
    },
)


class _EmailDefaultAttrs(_FromParam):
    id: str
    """
    The Email ID.
    """
    to: Union[List[str], str]
    """
    List of email addresses to send the email to.
    """
    created_at: str
    """
    When the email was created.
    """
    subject: str
    """
    The subject of the email.
    """
    html: str
    """
    The HTML content of the email.
    """
    text: str
    """
    The text content of the email.
    """
    bcc: Union[List[str], str]
    """
    Bcc
    """
    cc: Union[List[str], str]
    """
    Cc
    """
    reply_to: Union[List[str], str]
    """
    Reply to
    """
    last_event: str
    """
    The last event of the email.
    """


class Email(_EmailDefaultAttrs):
    """
    Email type that wraps the email object

    Attributes:
        id (str): The Email ID.
        from (str): The email address the email was sent from.
        to (Union[List[str], str]): List of email addresses to send the email to.
        created_at (str): When the email was created.
        subject (str): The subject of the email.
        html (str): The HTML content of the email.
        text (str): The text content of the email.
        bcc (Union[List[str], str]): Bcc
        cc (Union[List[str], str]): Cc
        reply_to (Union[List[str], str]): Reply to
        last_event (str): The last event of the email.
    """



from typing import List, Union

from typing_extensions import TypedDict

from resend.domains._record import Record


class Domain(TypedDict):
    id: str
    """
    The domain ID
    """
    name: str
    """
    The domain name
    """
    created_at: str
    """
    When domain was created
    """
    status: str
    """
    Status of the domain: not_started, etc..
    """
    region: str
    """
    The region where emails will be sent from. Possible values: us-east-1' | 'eu-west-1' | 'sa-east-1' | 'ap-northeast-1'
    """
    records: Union[List[Record], None]
    """
    The list of domain records
    """
    deleted: bool
    """
    Wether the domain is deleted or not
    """



from typing import Any, Dict, List, Union, cast

from typing_extensions import NotRequired, TypedDict

from resend import request

from ._broadcast import Broadcast

# _CreateParamsFrom is declared with functional TypedDict syntax here because
# "from" is a reserved keyword in Python, and this is the best way to
# support type-checking for it.
_CreateParamsFrom = TypedDict(
    "_CreateParamsFrom",
    {
        "from": str,
    },
)


class _CreateResponse(TypedDict):
    id: str
    """
    id of the created broadcast
    """


class _SendResponse(_CreateResponse):
    pass


class _RemoveResponse(TypedDict):
    object: str
    """
    object type: "broadcast"
    """
    id: str
    """
    id of the removed broadcast
    """
    deleted: bool
    """
    True if the broadcast was deleted
    """


class _ListResponse(TypedDict):
    object: str
    """
    object type: "list"
    """
    data: List[Broadcast]
    """
    A list of broadcast objects
    """


class _CreateParamsDefault(_CreateParamsFrom):
    audience_id: str
    """
    The ID of the audience you want to send to.
    """
    subject: str
    """
    Email subject.
    """
    reply_to: NotRequired[Union[List[str], str]]
    """
    Reply-to email address. For multiple addresses, send as an array of strings.
    """
    html: NotRequired[str]
    """
    The HTML version of the message.
    """
    text: NotRequired[str]
    """
    The text version of the message.
    """
    name: NotRequired[str]
    """
    The friendly name of the broadcast. Only used for internal reference.
    """


class _SendBroadcastParams(TypedDict):
    broadcast_id: str
    """
    The ID of the broadcast to send.
    """
    scheduled_at: NotRequired[str]
    """
    Schedule email to be sent later.
    The date should be in natural language (e.g.: in 1 min) or ISO 8601 format (e.g: 2024-08-05T11:52:01.858Z).
    """


class Broadcasts:

    class CreateParams(_CreateParamsDefault):
        """CreateParams is the class that wraps the parameters for the create method.

        Attributes:
            from (str): The sender email address
            audience_id (str): The ID of the audience you want to send to.
            subject (str): Email subject.
            reply_to (NotRequired[Union[List[str], str]]): Reply-to email address(es).
            html (NotRequired[str]): The HTML version of the message.
            text (NotRequired[str]): The text version of the message.
            name (NotRequired[str]): The friendly name of the broadcast. Only used for internal reference.
        """

    class SendParams(_SendBroadcastParams):
        """SendParams is the class that wraps the parameters for the send method.

        Attributes:
            broadcast_id (str): The ID of the broadcast to send.
            scheduled_at (NotRequired[str]): Schedule email to be sent later.
            The date should be in natural language (e.g.: in 1 min) or ISO 8601 format (e.g: 2024-08-05T11:52:01.858Z).
        """

    class CreateResponse(_CreateResponse):
        """
        CreateResponse is the class that wraps the response of the create method.

        Attributes:
            id (str): id of the created broadcast
        """

    class SendResponse(_SendResponse):
        """
        SendResponse is the class that wraps the response of the send method.

        Attributes:
            id (str): id of the created broadcast
        """

    class ListResponse(_ListResponse):
        """
        ListResponse is the class that wraps the response of the list method.

        Attributes:
            object (str): object type: "list"
            data (List[Broadcast]): A list of broadcast objects
        """

    class RemoveResponse(_RemoveResponse):
        """
        RemoveResponse is the class that wraps the response of the remove method.

        Attributes:
            object (str): object type: "broadcast"
            id (str): id of the removed broadcast
            deleted (bool): True if the broadcast was deleted
        """

    @classmethod
    def create(cls, params: CreateParams) -> CreateResponse:
        """
        Create a broadcast.
        see more: https://resend.com/docs/api-reference/broadcasts/create-broadcast

        Args:
            params (CreateParams): The broadcast creation parameters

        Returns:
            CreateResponse: The new broadcast object response
        """
        path = "/broadcasts"
        resp = request.Request[_CreateResponse](
            path=path, params=cast(Dict[Any, Any], params), verb="post"
        ).perform_with_content()
        return resp

    @classmethod
    def send(cls, params: SendParams) -> SendResponse:
        """
        Sends a broadcast.
        see more: https://resend.com/docs/api-reference/broadcasts/send-broadcast

        Args:
            params (CreateParams): The broadcast creation parameters

        Returns:
            SendResponse: The new broadcast object response
        """
        path = f"/broadcasts/{params['broadcast_id']}/send"
        resp = request.Request[_SendResponse](
            path=path, params=cast(Dict[Any, Any], params), verb="post"
        ).perform_with_content()
        return resp

    @classmethod
    def list(cls) -> ListResponse:
        """
        Retrieve a list of broadcasts.
        see more: https://resend.com/docs/api-reference/broadcasts/list-broadcasts

        Returns:
            ListResponse: A list of broadcast objects
        """
        path = "/broadcasts/"
        resp = request.Request[_ListResponse](
            path=path, params={}, verb="get"
        ).perform_with_content()
        return resp

    @classmethod
    def get(cls, id: str) -> Broadcast:
        """
        Retrieve a single broadcast.
        see more: https://resend.com/docs/api-reference/broadcasts/get-broadcast

        Args:
            id (str): The broadcast ID

        Returns:
            Broadcast: The broadcast object
        """
        path = f"/broadcasts/{id}"
        resp = request.Request[Broadcast](
            path=path, params={}, verb="get"
        ).perform_with_content()
        return resp

    @classmethod
    def remove(cls, id: str) -> RemoveResponse:
        """
        Delete a single broadcast.
        see more: https://resend.com/docs/api-reference/broadcasts/delete-broadcasts

        Args:
            id (str): The broadcast ID

        Returns:
            RemoveResponse: The remove response object
        """
        path = f"/broadcasts/{id}"
        resp = request.Request[_RemoveResponse](
            path=path, params={}, verb="delete"
        ).perform_with_content()
        return resp



from typing_extensions import TypedDict


class Audience(TypedDict):
    id: str
    """
    The unique identifier of the audience.
    """
    name: str
    """
    The name of the audience.
    """
    created_at: str
    """
    The date and time the audience was created.
    """
    deleted: bool
    """
    Wether the audience was deleted. Only returned on the "remove" call
    """



from typing import Any, Dict, List, cast

from typing_extensions import TypedDict

from resend import request

from ._email import Email
from ._emails import Emails


class _SendResponse(TypedDict):
    data: List[Email]
    """
    A list of email objects
    """


class Batch:

    class SendResponse(_SendResponse):
        """
        SendResponse type that wraps a list of email objects

        Attributes:
            data (List[Email]): A list of email objects
        """

    @classmethod
    def send(cls, params: List[Emails.SendParams]) -> SendResponse:
        """
        Trigger up to 100 batch emails at once.
        see more: https://resend.com/docs/api-reference/emails/send-batch-emails

        Args:
            params (List[Emails.SendParams]): The list of emails to send

        Returns:
            SendResponse: A list of email objects
        """
        path = "/emails/batch"

        resp = request.Request[_SendResponse](
            path=path, params=cast(List[Dict[Any, Any]], params), verb="post"
        ).perform_with_content()
        return resp



from typing_extensions import TypedDict


class Record(TypedDict):
    record: str
    """
    The domain record type, ie: SPF.
    """
    name: str
    """
    The domain record name.
    """
    type: str
    """
    The domain record type, ie: MX.
    """
    ttl: str
    """
    The domain record time to live.
    """
    status: str
    """
    The domain record status: not_started, etc..
    """
    value: str
    """
    The domain record value.
    """
    priority: int
    """
    The domain record priority.
    """



from typing import Any, Dict, List, Union, cast

from typing_extensions import NotRequired, TypedDict

from resend import request
from resend.emails._attachment import Attachment
from resend.emails._email import Email
from resend.emails._tag import Tag


class _UpdateParams(TypedDict):
    id: str
    """
    The ID of the email to update.
    """
    scheduled_at: NotRequired[str]
    """
    Schedule email to be sent later.
    The date should be in ISO 8601 format (e.g: 2024-08-05T11:52:01.858Z).
    """


class _UpdateEmailResponse(TypedDict):
    object: str
    """
    The object type: email
    """
    id: str
    """
    The ID of the scheduled email that was canceled.
    """


class _CancelScheduledEmailResponse(TypedDict):
    object: str
    """
    The object type: email
    """
    id: str
    """
    The ID of the scheduled email that was canceled.
    """


# SendParamsFrom is declared with functional TypedDict syntax here because
# "from" is a reserved keyword in Python, and this is the best way to
# support type-checking for it.
_SendParamsFrom = TypedDict(
    "_SendParamsFrom",
    {
        "from": str,
    },
)


class _SendParamsDefault(_SendParamsFrom):
    to: Union[str, List[str]]
    """
    List of email addresses to send the email to.
    """
    subject: str
    """
    The subject of the email.
    """
    bcc: NotRequired[Union[List[str], str]]
    """
    Bcc
    """
    cc: NotRequired[Union[List[str], str]]
    """
    Cc
    """
    reply_to: NotRequired[Union[List[str], str]]
    """
    Reply to
    """
    html: NotRequired[str]
    """
    The HTML content of the email.
    """
    text: NotRequired[str]
    """
    The text content of the email.
    """
    headers: NotRequired[Dict[str, str]]
    """
    Custom headers to be added to the email.
    """
    attachments: NotRequired[List[Attachment]]
    """
    List of attachments to be added to the email.
    """
    tags: NotRequired[List[Tag]]
    """
    List of tags to be added to the email.
    """
    scheduled_at: NotRequired[str]
    """
    Schedule email to be sent later.
    The date should be in ISO 8601 format (e.g: 2024-08-05T11:52:01.858Z).
    """


class Emails:

    class CancelScheduledEmailResponse(_CancelScheduledEmailResponse):
        """
        CancelScheduledEmailResponse is the type that wraps the response of the email that was canceled

        Attributes:
            object (str): The object type
            id (str): The ID of the scheduled email that was canceled
        """

    class UpdateEmailResponse(_UpdateEmailResponse):
        """
        UpdateEmailResponse is the type for the updated email response.

        Attributes:
            object (str): The object type
            id (str): The ID of the updated email.
        """

    class UpdateParams(_UpdateParams):
        """
        UpdateParams is the class that wraps the parameters for the update method.

        Attributes:
            id (str): The ID of the email to update.
            scheduled_at (NotRequired[str]): Schedule email to be sent later. \
            The date should be in ISO 8601 format (e.g: 2024-08-05T11:52:01.858Z).
        """

    class SendParams(_SendParamsDefault):
        """SendParams is the class that wraps the parameters for the send method.

        Attributes:
            from (str): The email address to send the email from.
            to (Union[str, List[str]]): List of email addresses to send the email to.
            subject (str): The subject of the email.
            bcc (NotRequired[Union[List[str], str]]): Bcc
            cc (NotRequired[Union[List[str], str]]): Cc
            reply_to (NotRequired[Union[List[str], str]]): Reply to
            html (NotRequired[str]): The HTML content of the email.
            text (NotRequired[str]): The text content of the email.
            headers (NotRequired[Dict[str, str]]): Custom headers to be added to the email.
            attachments (NotRequired[List[Attachment]]): List of attachments to be added to the email.
            tags (NotRequired[List[Tag]]): List of tags to be added to the email.
        """

    @classmethod
    def send(cls, params: SendParams) -> Email:
        """
        Send an email through the Resend Email API.
        see more: https://resend.com/docs/api-reference/emails/send-email

        Args:
            params (SendParams): The email parameters

        Returns:
            Email: The email object that was sent
        """
        path = "/emails"
        resp = request.Request[Email](
            path=path,
            params=cast(Dict[Any, Any], params),
            verb="post",
        ).perform_with_content()
        return resp

    @classmethod
    def get(cls, email_id: str) -> Email:
        """
        Retrieve a single email.
        see more: https://resend.com/docs/api-reference/emails/retrieve-email

        Args:
            email_id (str): The ID of the email to retrieve

        Returns:
            Email: The email object that was retrieved
        """
        path = f"/emails/{email_id}"
        resp = request.Request[Email](
            path=path,
            params={},
            verb="get",
        ).perform_with_content()
        return resp

    @classmethod
    def cancel(cls, email_id: str) -> CancelScheduledEmailResponse:
        """
        Cancel a scheduled email.
        see more: https://resend.com/docs/api-reference/emails/cancel-email

        Args:
            email_id (str): The ID of the scheduled email to cancel

        Returns:
            CancelScheduledEmailResponse: The response object that contains the ID of the scheduled email that was canceled
        """
        path = f"/emails/{email_id}/cancel"
        resp = request.Request[_CancelScheduledEmailResponse](
            path=path,
            params={},
            verb="post",
        ).perform_with_content()
        return resp

    @classmethod
    def update(cls, params: UpdateParams) -> UpdateEmailResponse:
        """
        Update an email.
        see more: https://resend.com/docs/api-reference/emails/update-email

        Args:
            params (UpdateParams): The email parameters to update

        Returns:
            Email: The email object that was updated
        """
        path = f"/emails/{params['id']}"
        resp = request.Request[_UpdateEmailResponse](
            path=path,
            params=cast(Dict[Any, Any], params),
            verb="patch",
        ).perform_with_content()
        return resp



from typing_extensions import TypedDict


class ApiKey(TypedDict):
    id: str
    """
    The api key ID
    """
    token: str
    """
    The api key token
    """
    name: str
    """
    The api key token
    """
    created_at: str
    """
    Api key creation date
    """
